require_relative 'player'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
  class Game
    attr_reader :title

    def initialize(title)
      @title = title
      @players = []
    end

    def add_player(player)
      @players << player
    end

    def load_players(from_file)
      lines = File.readlines(from_file).each do |line|
        add_player(Player.from_csv(line))
      end
    end

    def play(rounds)
      puts "\nHere are the players:"
      @players.each do |player|
        puts player.name
      end

      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.length} treasures to be found."
      treasures.each do |treasure|
        puts "A #{treasure.name} is worth #{treasure.points} points."
      end

      1.upto(rounds) do |round|
        if block_given?
          break if yield
        end
        puts "\nRound Number #{round}"
        @players.each do |player|
          GameTurn.take_turn(player)
          puts player
        end
      end
    end

    def high_score_entry(player)
      "#{player.name.ljust(15, '.')} #{player.score}"
    end

    def print_stats
      strong, wimpy = @players.partition { |player| player.strong? }

      puts "\n#{@title} Statistics:\n"

      puts "#{strong.length} strong player(s):"
      strong.each do |player|
        print_name_and_health(player)
      end

      puts "\n#{wimpy.length} wimpy player(s):"
      wimpy.each do |player|
        print_name_and_health(player)
      end

      sorted_players = @players.sort

      puts "\n#{@title} High Scores:"
      sorted_players.each do |player|
        puts high_score_entry(player)
      end

      @players.each do |player|
        puts "\n#{player.name}'s point totals:"
        puts "#{player.points} grand total points"
      end

      puts "\n#{total_points} total points from treasures found"

      @players.sort.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{player.points} grand total points"
      end
    end

    def total_points
      @players.reduce(0) { |sum, player| sum + player.points }
    end

    def print_name_and_health (player)
      puts "#{player.name} (#{player.health})"
    end

    def save_high_scores(to_file = "high_scores.csv")
      File.open(to_file, "w") do |file|
        file.puts "Knuckleheads High Scores:"
        @players.sort.each do |player|
          file.puts high_score_entry(player)
        end
      end
    end
  end
end
